@@ -2,7 +2,7 @@
 #
 # Proxy class for reading YAML configuration
 #
-# Written by Scott Hardin, Martin Bartosch and Oliver Welter 
+# Written by Scott Hardin, Martin Bartosch and Oliver Welter
 # for the OpenXPKI project 2012
 #
 # THIS IS NOT WORKING IN A FORKING ENVIRONMENT!
@@ -27,12 +27,17 @@ sub _build_config {
 
 
 sub _get_node {
-    
+
     my $self = shift;
     my @path = $self->_build_path_with_prefix( shift );
 
     my $ptr = $self->_config();
 
+    # Top Level Node requested
+    if (!@path) {
+        return $ptr;
+    }
+
     while ( scalar @path > 1 ) {
         my $entry = shift @path;
         if ( exists $ptr->{$entry} ) {
@@ -46,130 +51,142 @@ sub _get_node {
             return $self->_node_not_exists($entry);
         }
     }
-    
+
     return $ptr->{ shift @path };
-    
+
 }
 
 sub get {
-    
-    my $self = shift;    
+
+    my $self = shift;
     my $value = $self->_get_node( shift );
-    
-    return undef unless (defined $value);
-    
+
+    return unless (defined $value);
+
     if (ref $value ne '') {
         die "requested value is not a scalar " . Dumper $value;
     }
-    
+
     return $value;
-    
+
 }
 
 sub get_size {
-    
-    my $self = shift;    
+
+    my $self = shift;
     my $node = $self->_get_node( shift );
-    
-    return undef unless(defined $node);
-    
+
+    return unless(defined $node);
+
     if ( ref $node ne 'ARRAY' ) {
         die "requested value is not a list"
     }
-    
-    return scalar @{$node};    
+
+    return scalar @{$node};
 }
 
 
 sub get_list {
-    
-    my $self = shift;    
+
+    my $self = shift;
     my $node = $self->_get_node( shift );
-    
-    return undef unless(defined $node);
-    
+
+    return unless(defined $node);
+
     if ( ref $node ne 'ARRAY' ) {
         die "requested value is not a list"
     }
-    
+
     return @{$node};
 }
 
 sub get_keys {
-    
-    my $self = shift;    
+
+    my $self = shift;
     my $node = $self->_get_node( shift );
-    
-    return undef unless(defined $node);
-    
+
+    return unless(defined $node);
+
     if ( ref $node ne 'HASH' ) {
         die "requested value is not a hash"
     }
-    
-    return keys %{$node};   
+
+    return keys %{$node};
 }
 
 sub get_hash {
-    
-    my $self = shift;    
+
+    my $self = shift;
     my $node = $self->_get_node( shift );
-    
-    return undef unless(defined $node);
-    
+
+    return unless(defined $node);
+
     if ( ref $node ne 'HASH' ) {
         die "requested value is not a hash"
     }
-    
-    return $node;   
-} 
+
+    return $node;
+}
 
 sub get_meta {
-    
+
     my $self = shift;
-    
+
     my $node = $self->_get_node( shift );
 
     if (!defined $node) {
-        return undef;
+        # die_on_undef already handled by get_node
+        return;
     }
 
     my $meta = {};
 
     if (ref $node eq '') {
-        $meta = {TYPE  => "scalar", VALUE => $node };        
+        $meta = {TYPE  => "scalar", VALUE => $node };
     } elsif (ref $node eq "SCALAR") {
-        $meta = {TYPE  => "reference", VALUE => $$node };        
+        $meta = {TYPE  => "reference", VALUE => $$node };
     } elsif (ref $node eq "ARRAY") {
         $meta = {TYPE  => "list", VALUE => $node };
     } elsif (ref $node eq "HASH") {
         my @keys = keys(%{$node});
-        $meta = {TYPE  => "hash", VALUE => \@keys };        
+        $meta = {TYPE  => "hash", VALUE => \@keys };
     } else {
         die "Unsupported node type";
-    }    
+    }
     return $meta;
 }
 
+sub exists {
+
+    my $self = shift;
+
+    my $value = 0;
+    eval {
+        $value = defined $self->_get_node( shift );
+    };
+    return $value;
+
+}
 
 sub set {
-    
+
     my $self = shift;
     my @path = $self->_build_path_with_prefix( shift );
 
     my $value = shift;
 
     my $ptr = $self->_config();
-    
+
     while (scalar @path > 1) {
-        my $entry = shift @path;        
+        my $entry = shift @path;
         if (!exists $ptr->{$entry}) {
             $ptr->{$entry} = {};
         } elsif (ref $ptr->{$entry} ne "HASH") {
             confess('Try to step over a value node at ' . $entry);
         }
-        $ptr = $ptr->{$entry};    
+        $ptr = $ptr->{$entry};
     }
-    
+
     my $entry = shift @path;
 
     if (exists $ptr->{$entry}) {
@@ -194,3 +211,4 @@ Connector::Builtin::Memory
 =head 1 Description
 
 A connector implementation to allow memory based caching
+
